searchNodes=[{"doc":"A utility for writing dynamic queries.It allows getting rid of some boilerplate when building a query based on input parameters.params = %{search_query: &quot;John Doe&quot;} User |&gt; where(active: true) |&gt; Composite.new(params) |&gt; Composite.param(:org_id, &amp;filter_by_org_id/2) |&gt; Composite.param(:search_query, &amp;search_by_full_name/2) |&gt; Composite.param(:org_name, &amp;filter_by_org_name/2, requires: :org) |&gt; Composite.param(:org_type, &amp;filter_by_org_type/2, requires: :org) |&gt; Composite.dependency(:org, &amp;join_orgs/1) |&gt; Repo.all()Even though most of the examples in this doc use Ecto, Composite itself is not limited only to it. Ecto is an optional dependency and it is present only for having an implementation of Ecto.Queryable OOTB. You're able to use Composite with any Elixir term, as it is just an advanced wrapper around Enum.reduce/3.","ref":"Composite.html","title":"Composite","type":"module"},{"doc":"Applies handlers to query.Used when composite is defined with new/2.If used with Ecto, then calling this function is not necessary, as Composite implements Ecto.Queryable protocol, so applying will be done automatically when it is needed.","ref":"Composite.html#apply/1","title":"Composite.apply/1","type":"function"},{"doc":"Applies handlers to query.Used when composite is defined with new/0","ref":"Composite.html#apply/3","title":"Composite.apply/3","type":"function"},{"doc":"Defines a dependency loader.Dependency is an instruction which is being applied lazily to a query. The same dependency can be required by many parameters, but it will be invoked only once. Dependency can depend on other dependency.Useful for joining tables.params = %{org_type: :nonprofit, is_org_closed: false} User |&gt; Composite.new(params) |&gt; Composite.param(:is_org_closed, &amp;where(&amp;1, [orgs: orgs], orgs.closed == ^&amp;2), requires: :orgs) |&gt; Composite.param(:org_type, &amp;where(&amp;1, [orgs: orgs], orgs.type == ^&amp;2), requires: :orgs) |&gt; Composite.dependency(:orgs, &amp;join(&amp;1, :inner, [users], orgs in assoc(users, :org), as: :orgs))It is also possible to require a dependency only if specific value is set. In example below dependency :phone will be loaded only if value of :search param starts from + signcomposite |&gt; Composite.param( :search, fn query, &quot;+&quot; &lt;&gt; _ = phone_number -&gt; where(query, [phones: phones], phones.number == ^phone_number) query, query_string -&gt; where(query, [records], ilike(records.text, ^query_string)) end, requires: fn &quot;+&quot; &lt;&gt; _ -&gt; :phone _ -&gt; nil end ) |&gt; Composite.dependency(:phone, &amp;join(&amp;1, :inner, [records], phones in assoc(records, :phone), as: :phones))When loader function has arity 2, then all parameters are passed in the second argument.Options:requires - allows to set dependencies for current dependency.","ref":"Composite.html#dependency/4","title":"Composite.dependency/4","type":"function"},{"doc":"Forces loading dependency even if it is not required by params.","ref":"Composite.html#force_require/2","title":"Composite.force_require/2","type":"function"},{"doc":"Initializes a Composite struct with delayed application of query and params.Must be used with apply/3.composite = Composite.new() |&gt; Composite.param(:organization_id, &amp;where(&amp;1, organization_id: ^&amp;2)) |&gt; Composite.param(:age_more_than, &amp;where(&amp;1, [users], users.age &gt; ^&amp;2)) params = %{organization_id: 1} User |&gt; where(active: true) |&gt; Composite.apply(composite, params) |&gt; Repo.all()","ref":"Composite.html#new/0","title":"Composite.new/0","type":"function"},{"doc":"Initializes a Composite struct with query and params.Must be used with apply/1.This strategy is useful when working with Ecto.Query in pipe-based queries.params = %{organization_id: 1} User |&gt; where(active: true) |&gt; Composite.new(params) |&gt; Composite.param(:organization_id, &amp;where(&amp;1, organization_id: ^&amp;2)) |&gt; Composite.param(:age_more_than, &amp;where(&amp;1, [users], users.age &gt; ^&amp;2)) |&gt; Repo.all()Please note, that there is no explicit Composite.apply/1 call before Repo.all/1, because Composite implements Ecto.Queryable protocol.","ref":"Composite.html#new/2","title":"Composite.new/2","type":"function"},{"doc":"Defines a parameter handler.Handler is applied to a query when apply/1 or apply/3 is invoked. All handlers are invoke in the same order as they are defined.If the parameter requires dependencies, then they will be loaded before the parameters' handler and only if parameter wasn't ignored. Examples with dependencies usage can be found in doc for dependency/4params = %{location: &quot;Arctic&quot;, order: :age_desc} User |&gt; Composite.new(params) |&gt; Composite.param(:location, &amp;where(&amp;1, location: ^&amp;2), ignore?: &amp;(&amp;1 in [nil, &quot;WORLDWIDE&quot;, &quot;&quot;]) ) |&gt; Composite.param( :order, fn query, :name_asc -&gt; query |&gt; order_by(name: :asc) query, :age_desc -&gt; query |&gt; order_by(age: :desc) end, on_ignore: &amp;order_by(&amp;1, inserted_at: :desc) )If input parameters have nested maps (or any other key-based data structure):params = %{filter: %{name: &quot;John&quot;}} User |&gt; Composite.new(params) |&gt; Composite.param([:filter, :name], &amp;where(&amp;1, name: ^&amp;2))Options:ignore? - if function returns true, then handler apply_fun/1 won't be applied. Default value is &amp;(&amp;1 in [nil, &quot;&quot;, [], %{}]).:on_ignore - a function that will be applied instead of apply_fun/1 if value is ignored. Defaults to Function.identity/1.:requires - points to the dependencies which has to be loaded before calling apply_fun/1. It is, also, possible to specify dependencies dynamically based on a value of the parameter by passing a function. The latter function will always receive not ignored values. Defaults to nil (which is equivalent to []).:ignore_requires - points to the dependencies which has to be loaded when value is ignored. May be needed for custom :on_ignore implementation.","ref":"Composite.html#param/4","title":"Composite.param/4","type":"function"},{"doc":"","ref":"Composite.html#t:apply_fun/1","title":"Composite.apply_fun/1","type":"type"},{"doc":"","ref":"Composite.html#t:dependencies/0","title":"Composite.dependencies/0","type":"type"},{"doc":"","ref":"Composite.html#t:dependency_name/0","title":"Composite.dependency_name/0","type":"type"},{"doc":"","ref":"Composite.html#t:dependency_option/0","title":"Composite.dependency_option/0","type":"type"},{"doc":"","ref":"Composite.html#t:load_dependency/1","title":"Composite.load_dependency/1","type":"type"},{"doc":"","ref":"Composite.html#t:param_option/1","title":"Composite.param_option/1","type":"type"},{"doc":"","ref":"Composite.html#t:param_path_item/0","title":"Composite.param_path_item/0","type":"type"},{"doc":"","ref":"Composite.html#t:params/0","title":"Composite.params/0","type":"type"},{"doc":"","ref":"Composite.html#t:t/1","title":"Composite.t/1","type":"type"}]